@@ -525,7 +525,7 @@ poll_more:
 		if (unlikely(ib_req_notify_cq(priv->recv_cq,
 					      IB_CQ_NEXT_COMP |
 					      IB_CQ_REPORT_MISSED_EVENTS)) &&
-		    napi_reschedule(napi))
+		    napi_schedule(napi))
 			goto poll_more;
 	}
 
@@ -555,7 +555,7 @@ poll_more:
 		napi_complete(napi);
 		if (unlikely(ib_req_notify_cq(priv->send_cq, IB_CQ_NEXT_COMP |
 					      IB_CQ_REPORT_MISSED_EVENTS)) &&
-		    napi_reschedule(napi))
+		    napi_schedule(napi))
 			goto poll_more;
 	}
 	return n < 0 ? 0 : n;
@@ -576,7 +576,7 @@ void ipoib_napi_schedule_work(struct work_struct *work)
 	bool ret;
 
 	do {
-		ret = napi_reschedule(&priv->send_napi);
+		ret = napi_schedule(&priv->send_napi);
 		if (!ret)
 			msleep(3);
 	} while (!ret && netif_queue_stopped(priv->dev) &&
@@ -588,7 +588,7 @@ void ipoib_ib_tx_completion(struct ib_cq *cq, void *ctx_ptr)
 	struct ipoib_dev_priv *priv = ctx_ptr;
 	bool ret;
 
-	ret = napi_reschedule(&priv->send_napi);
+	ret = napi_schedule(&priv->send_napi);
 	/*
 	 * if the queue is closed the driver must be able to schedule napi,
 	 * otherwise we can end with closed queue forever, because no new
@@ -916,7 +916,7 @@ int ipoib_ib_dev_stop_default(struct net_device *dev)
 timeout:
 	qp_attr.qp_state = IB_QPS_RESET;
 	if (ib_modify_qp(priv->qp, &qp_attr, IB_QP_STATE))
-		check_qp_movement_and_print(priv, priv->qp, IB_QPS_RESET);
+		ipoib_warn(priv, "Failed to modify QP to RESET state\n");
 
 	ib_req_notify_cq(priv->recv_cq, IB_CQ_NEXT_COMP);
 
