@@ -314,10 +314,6 @@ static const struct devlink_ops mlx5_devlink_ops = {
 #ifdef CONFIG_MLX5_ESWITCH
 	.eswitch_mode_set = mlx5_devlink_eswitch_mode_set,
 	.eswitch_mode_get = mlx5_devlink_eswitch_mode_get,
-#ifdef HAVE_DEVLINK_ESWITCH_STATE
-	.eswitch_state_get = mlx5_devlink_eswitch_state_get,
-	.eswitch_state_set = mlx5_devlink_eswitch_state_set,
-#endif
 	.eswitch_inline_mode_set = mlx5_devlink_eswitch_inline_mode_set,
 	.eswitch_inline_mode_get = mlx5_devlink_eswitch_inline_mode_get,
 	.eswitch_encap_mode_set = mlx5_devlink_eswitch_encap_mode_set,
