@@ -307,8 +307,7 @@ struct mlx5_eswitch_fdb {
 			struct mlx5_flow_namespace *ns;
 			struct mlx5_flow_table *drop_root;
 			struct mlx5_flow_handle *drop_root_rule;
-			struct mlx5_fc *drop_root_counter;
-			struct dentry *drop_root_dbgfs;
+			struct mlx5_fc *drop_root_fc;
 			struct mlx5_flow_table *tc_miss_table;
 			struct mlx5_flow_table *slow_fdb;
 			struct mlx5_flow_group *send_to_vport_grp;
@@ -459,9 +458,7 @@ struct mlx5_eswitch {
 	struct mlx5_esw_offload offloads;
 	u32 last_vport_idx; /* ++ every time a vport is created */
 	int                     mode;
-#ifdef HAVE_DEVLINK_ESWITCH_STATE
-	u8                      state;
-#endif
+	bool                    offloads_inactive;
 	u16                     manager_vport;
 	u16                     first_host_vport;
 	u8			num_peers;
@@ -675,16 +672,6 @@ int mlx5_devlink_eswitch_encap_mode_set(struct devlink *devlink,
 					struct netlink_ext_ack *extack);
 int mlx5_devlink_eswitch_encap_mode_get(struct devlink *devlink,
 					enum devlink_eswitch_encap_mode *encap);
-/* Added for compilation on 6.15 base kernel */
-#ifdef HAVE_DEVLINK_ESWITCH_STATE
-int mlx5_devlink_eswitch_state_get(struct devlink *devlink,
-				   enum devlink_eswitch_state *state);
-#endif
-#ifdef HAVE_DEVLINK_ESWITCH_STATE
-int mlx5_devlink_eswitch_state_set(struct devlink *devlink,
-				   enum devlink_eswitch_state state,
-				   struct netlink_ext_ack *extack);
-#endif
 int mlx5_devlink_port_fn_hw_addr_get(struct devlink_port *port,
 				     u8 *hw_addr, int *hw_addr_len,
 				     struct netlink_ext_ack *extack);
