@@ -353,7 +353,7 @@ static u16 nvmet_rdma_install_offload_queue(struct nvmet_sq *sq)
 		if (ret) {
 			pr_err("failed to get XRQ for queue (%d)\n",
 			       queue->host_qid);
-			return NVME_SC_INTERNAL | NVME_SC_DNR;
+			return NVME_SC_INTERNAL | NVME_STATUS_DNR;
 		}
 		qp_attr_mask |= IB_QP_RMPN_XRQN;
 		attr.rmpn_xrqn = queue->xrq->ofl_srq->srq->ext.xrc.srq_num;
@@ -361,7 +361,7 @@ static u16 nvmet_rdma_install_offload_queue(struct nvmet_sq *sq)
 
 	ret = ib_modify_qp(queue->cm_id->qp, &attr, qp_attr_mask);
 	if (ret)
-		return NVME_SC_INTERNAL | NVME_SC_DNR;
+		return NVME_SC_INTERNAL | NVME_STATUS_DNR;
 	return 0;
 }
 
