@@ -11,6 +11,9 @@
 #include <linux/init.h>
 #include <linux/slab.h>
 #include <linux/err.h>
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+#include <linux/crc32.h>
+#endif
 #include <linux/nvme-tcp.h>
 #ifdef CONFIG_NVME_TCP_TLS
 #include <linux/nvme-keyring.h>
@@ -23,7 +26,9 @@
 #include <net/handshake.h>
 #endif
 #include <linux/blk-mq.h>
+#ifndef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
 #include <crypto/hash.h>
+#endif
 #include <net/busy_poll.h>
 #include <trace/events/sock.h>
 #include <linux/sched/mm.h>
@@ -176,8 +181,13 @@ struct nvme_tcp_queue {
 	bool			hdr_digest;
 	bool			data_digest;
 	bool			tls_enabled;
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+	u32			rcv_crc;
+	u32			snd_crc;
+#else
 	struct ahash_request	*rcv_hash;
 	struct ahash_request	*snd_hash;
+#endif
 	__le32			exp_ddgst;
 	__le32			recv_ddgst;
 	struct completion       tls_complete;
@@ -480,6 +490,41 @@ nvme_tcp_fetch_request(struct nvme_tcp_queue *queue)
 	return req;
 }
 
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+#define NVME_TCP_CRC_SEED (~0)
+
+static inline void nvme_tcp_ddgst_update(u32 *crcp,
+		struct page *page, size_t off, size_t len)
+{
+	page += off / PAGE_SIZE;
+	off %= PAGE_SIZE;
+	while (len) {
+		const void *vaddr = kmap_local_page(page);
+		size_t n = min(len, (size_t)PAGE_SIZE - off);
+
+		*crcp = crc32c(*crcp, vaddr + off, n);
+		kunmap_local(vaddr);
+		page++;
+		off = 0;
+		len -= n;
+	}
+}
+
+static inline __le32 nvme_tcp_ddgst_final(u32 crc)
+{
+	return cpu_to_le32(~crc);
+}
+
+static inline __le32 nvme_tcp_hdgst(const void *pdu, size_t len)
+{
+	return cpu_to_le32(~crc32c(NVME_TCP_CRC_SEED, pdu, len));
+}
+
+static inline void nvme_tcp_set_hdgst(void *pdu, size_t len)
+{
+	*(__le32 *)(pdu + len) = nvme_tcp_hdgst(pdu, len);
+}
+#else
 static inline void nvme_tcp_ddgst_final(struct ahash_request *hash,
 		__le32 *dgst)
 {
@@ -507,6 +552,7 @@ static inline void nvme_tcp_hdgst(struct ahash_request *hash,
 	ahash_request_set_crypt(hash, &sg, pdu + len, len);
 	crypto_ahash_digest(hash);
 }
+#endif
 
 static int nvme_tcp_verify_hdgst(struct nvme_tcp_queue *queue,
 		void *pdu, size_t pdu_len)
@@ -523,8 +569,12 @@ static int nvme_tcp_verify_hdgst(struct nvme_tcp_queue *queue,
 	}
 
 	recv_digest = *(__le32 *)(pdu + hdr->hlen);
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+	exp_digest = nvme_tcp_hdgst(pdu, pdu_len);
+#else
 	nvme_tcp_hdgst(queue->rcv_hash, pdu, pdu_len);
 	exp_digest = *(__le32 *)(pdu + hdr->hlen);
+#endif
 	if (recv_digest != exp_digest) {
 		dev_err(queue->ctrl->ctrl.device,
 			"header digest error: recv %#x expected %#x\n",
@@ -550,7 +600,11 @@ static int nvme_tcp_check_ddgst(struct nvme_tcp_queue *queue, void *pdu)
 		nvme_tcp_queue_id(queue));
 		return -EPROTO;
 	}
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+	queue->rcv_crc = NVME_TCP_CRC_SEED;
+#else
 	crypto_ahash_init(queue->rcv_hash);
+#endif
 
 	return 0;
 }
@@ -954,8 +1008,13 @@ static int nvme_tcp_recv_data(struct nvme_tcp_queue *queue, struct sk_buff *skb,
 				iov_iter_count(&req->iter));
 
 		if (queue->data_digest)
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+			ret = skb_copy_and_crc32c_datagram_iter(skb, *offset,
+				&req->iter, recv_len, &queue->rcv_crc);
+#else
 			ret = skb_copy_and_hash_datagram_iter(skb, *offset,
 				&req->iter, recv_len, queue->rcv_hash);
+#endif
 		else
 			ret = skb_copy_datagram_iter(skb, *offset,
 					&req->iter, recv_len);
@@ -973,7 +1032,11 @@ static int nvme_tcp_recv_data(struct nvme_tcp_queue *queue, struct sk_buff *skb,
 
 	if (!queue->data_remaining) {
 		if (queue->data_digest) {
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+			queue->exp_ddgst = nvme_tcp_ddgst_final(queue->rcv_crc);
+#else
 			nvme_tcp_ddgst_final(queue->rcv_hash, &queue->exp_ddgst);
+#endif
 			queue->ddgst_remaining = NVME_TCP_DIGEST_LENGTH;
 		} else {
 			if (pdu->hdr.flags & NVME_TCP_F_DATA_SUCCESS) {
@@ -1200,8 +1263,13 @@ static int nvme_tcp_try_send_data(struct nvme_tcp_request *req)
 			return ret;
 
 		if (queue->data_digest)
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+			nvme_tcp_ddgst_update(&queue->snd_crc, page,
+					offset, ret);
+#else
 			nvme_tcp_ddgst_update(queue->snd_hash, page,
 					offset, ret);
+#endif
 
 		/*
 		 * update the request iterator except for the last payload send
@@ -1214,8 +1282,13 @@ static int nvme_tcp_try_send_data(struct nvme_tcp_request *req)
 		/* fully successful last send in current PDU */
 		if (last && ret == len) {
 			if (queue->data_digest) {
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+				req->ddgst =
+					nvme_tcp_ddgst_final(queue->snd_crc);
+#else
 				nvme_tcp_ddgst_final(queue->snd_hash,
 					&req->ddgst);
+#endif
 				req->state = NVME_TCP_SEND_DDGST;
 				req->offset = 0;
 			} else {
@@ -1258,7 +1331,11 @@ static int nvme_tcp_try_send_cmd_pdu(struct nvme_tcp_request *req)
 #endif
 
 	if (queue->hdr_digest && !req->offset)
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+		nvme_tcp_set_hdgst(pdu, sizeof(*pdu));
+#else
 		nvme_tcp_hdgst(queue->snd_hash, pdu, sizeof(*pdu));
+#endif
 
 #ifdef HAVE_PROTO_OPS_SENDPAGE
 	ret = kernel_sendpage(queue->sock, virt_to_page(pdu),
@@ -1276,7 +1353,11 @@ static int nvme_tcp_try_send_cmd_pdu(struct nvme_tcp_request *req)
 		if (inline_data) {
 			req->state = NVME_TCP_SEND_DATA;
 			if (queue->data_digest)
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+				queue->snd_crc = NVME_TCP_CRC_SEED;
+#else
 				crypto_ahash_init(queue->snd_hash);
+#endif
 		} else {
 			nvme_tcp_done_send_req(queue);
 		}
@@ -1300,7 +1381,11 @@ static int nvme_tcp_try_send_data_pdu(struct nvme_tcp_request *req)
 	int ret;
 
 	if (queue->hdr_digest && !req->offset)
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+		nvme_tcp_set_hdgst(pdu, sizeof(*pdu));
+#else
 		nvme_tcp_hdgst(queue->snd_hash, pdu, sizeof(*pdu));
+#endif
 
 	if (!req->h2cdata_left)
 #ifdef HAVE_PROTO_OPS_SENDPAGE
@@ -1325,7 +1410,11 @@ static int nvme_tcp_try_send_data_pdu(struct nvme_tcp_request *req)
 	if (!len) {
 		req->state = NVME_TCP_SEND_DATA;
 		if (queue->data_digest)
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+			queue->snd_crc = NVME_TCP_CRC_SEED;
+#else
 			crypto_ahash_init(queue->snd_hash);
+#endif
 		return 1;
 	}
 	req->offset += ret;
@@ -1465,6 +1554,7 @@ static void nvme_tcp_io_work(struct work_struct *w)
 	queue_work_on(queue->io_cpu, nvme_tcp_wq, &queue->io_work);
 }
 
+#ifndef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
 static void nvme_tcp_free_crypto(struct nvme_tcp_queue *queue)
 {
 	struct crypto_ahash *tfm = crypto_ahash_reqtfm(queue->rcv_hash);
@@ -1499,6 +1589,7 @@ free_tfm:
 	crypto_free_ahash(tfm);
 	return -ENOMEM;
 }
+#endif
 
 static void nvme_tcp_free_async_req(struct nvme_tcp_ctrl *ctrl)
 {
@@ -1535,8 +1626,10 @@ static void nvme_tcp_free_queue(struct nvme_ctrl *nctrl, int qid)
 	if (!test_and_clear_bit(NVME_TCP_Q_ALLOCATED, &queue->flags))
 		return;
 
+#ifndef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
 	if (queue->hdr_digest || queue->data_digest)
 		nvme_tcp_free_crypto(queue);
+#endif
 
 #ifdef HAVE_PAGE_FRAG_CACHE_DRAIN
 	page_frag_cache_drain(&queue->pf_cache);
@@ -2020,6 +2113,7 @@ static int nvme_tcp_alloc_queue(struct nvme_ctrl *nctrl, int qid,
 
 	queue->hdr_digest = nctrl->opts->hdr_digest;
 	queue->data_digest = nctrl->opts->data_digest;
+#ifndef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
 	if (queue->hdr_digest || queue->data_digest) {
 		ret = nvme_tcp_alloc_crypto(queue);
 		if (ret) {
@@ -2028,13 +2122,18 @@ static int nvme_tcp_alloc_queue(struct nvme_ctrl *nctrl, int qid,
 			goto err_sock;
 		}
 	}
+#endif
 
 	rcv_pdu_size = sizeof(struct nvme_tcp_rsp_pdu) +
 			nvme_tcp_hdgst_len(queue);
 	queue->pdu = kmalloc(rcv_pdu_size, GFP_KERNEL);
 	if (!queue->pdu) {
 		ret = -ENOMEM;
+#ifdef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
+		goto err_sock;
+#else
 		goto err_crypto;
+#endif
 	}
 
 	dev_dbg(nctrl->device, "connecting queue %d\n",
@@ -2067,9 +2166,11 @@ err_init_connect:
 	kernel_sock_shutdown(queue->sock, SHUT_RDWR);
 err_rcv_pdu:
 	kfree(queue->pdu);
+#ifndef HAVE_SKB_COPY_AND_CRC32C_DATAGRAM_ITER
 err_crypto:
 	if (queue->hdr_digest || queue->data_digest)
 		nvme_tcp_free_crypto(queue);
+#endif
 err_sock:
 	/* ->sock will be released by fput() */
 	fput(queue->sock->file);
@@ -3227,7 +3328,7 @@ static struct nvmf_transport_ops nvme_tcp_transport = {
 			  NVMF_OPT_TLS |
 #ifdef HAVE_SOCK_SETOPTVAL_SOCKPTR_T
 			  NVMF_OPT_HOST_IFACE |
-#endif			  
+#endif
 			  NVMF_OPT_KEYRING | NVMF_OPT_TLS_KEY | NVMF_OPT_CONCAT,
 	.create_ctrl	= nvme_tcp_create_ctrl,
 };
