@@ -1002,6 +1002,7 @@ typedef void (*mlx5_cmd_cbk_t)(int status, void *context);
 enum {
 	MLX5_CMD_ENT_STATE_PENDING_COMP,
 	MLX5_CMD_ENT_STATE_RREF_CLEARED,
+	MLX5_CMD_ENT_STATE_TIMEDOUT,
 };
 
 struct mlx5_cmd_work_ent {
@@ -1543,6 +1544,13 @@ enum {
 
 bool mlx5_wc_support_get(struct mlx5_core_dev *mdev);
 
+static inline struct net *mlx5_core_net(struct mlx5_core_dev *dev)
+{
+	return devlink_net(priv_to_devlink(dev));
+}
+
+#define MLX5_SW_IMAGE_GUID_MAX_BYTES 9
+
 /* MLX5 Diagnostics */
 
 #define MLX5_DIAG_DUMP_VERSION  1
