@@ -181,11 +181,7 @@ enum devlink_sb_threshold_type {
 enum devlink_eswitch_mode {
 	DEVLINK_ESWITCH_MODE_LEGACY,
 	DEVLINK_ESWITCH_MODE_SWITCHDEV,
-};
-
-enum devlink_eswitch_state {
-	DEVLINK_ESWITCH_STATE_INACTIVE,
-	DEVLINK_ESWITCH_STATE_ACTIVE
+	DEVLINK_ESWITCH_MODE_SWITCHDEV_INACTIVE,
 };
 
 enum devlink_eswitch_inline_mode {
@@ -641,10 +637,6 @@ enum devlink_attr {
 
 	DEVLINK_ATTR_RATE_TC_BWS,		/* nested */
 
-	DEVLINK_ATTR_HEALTH_REPORTER_BURST_PERIOD,	/* u64 */
-
-	DEVLINK_ATTR_ESWITCH_STATE,	/* u8 */
-
 	/* Add new attributes above here, update the spec in
 	 * Documentation/netlink/specs/devlink.yaml and re-generate
 	 * net/devlink/netlink_gen.c.
