@@ -81,7 +81,11 @@ MODULE_INFO(retpoline, "Y");
 #endif
 MODULE_AUTHOR("Alex Nezhinsky, Dan Bar Dov, Or Gerlitz");
 
-static const struct scsi_host_template iscsi_iser_sht;
+static
+#ifdef HAVE_ISCSI_HOST_ALLOC_GET_CONST
+const
+#endif
+struct scsi_host_template iscsi_iser_sht;
 static struct iscsi_transport iscsi_iser_transport;
 static struct scsi_transport_template *iscsi_iser_scsi_transport;
 static struct workqueue_struct *release_wq;
@@ -995,7 +999,11 @@ static int iscsi_iser_slave_alloc(struct scsi_device *sdev)
 }
 #endif
 
-static const struct scsi_host_template iscsi_iser_sht = {
+static
+#ifdef HAVE_ISCSI_HOST_ALLOC_GET_CONST
+const
+#endif
+struct scsi_host_template iscsi_iser_sht = {
 	.module                 = THIS_MODULE,
 	.name                   = "iSCSI Initiator over iSER",
 	.queuecommand           = iscsi_queuecommand,
