@@ -146,25 +146,28 @@ conntrack_offload_del_conn(struct conntrack *ct,
          * checking.
          */
         if (flush) {
-            /* For flush, keep only terminated flag, so further TCP packets
+            /* For flush, keep only bypass flag, so further TCP packets
              * will still bypass seq checking.
              */
-            conn_dir->offloads.flags = CT_OFFLOAD_TERMINATED;
+            conn_dir->offloads.flags = CT_OFFLOAD_BYPASS_TCP_SEQ_CHK;
         } else {
-            conn_dir->offloads.flags |= CT_OFFLOAD_TERMINATED | CT_OFFLOAD_SKIP_BOTH;
+            conn_dir->offloads.flags |= CT_OFFLOAD_SKIP_BOTH | CT_OFFLOAD_DELETED;
         }
         if (dir == CT_DIR_REP) {
             conn_dir->offloads.flags &= ~CT_OFFLOAD_REP;
         } else {
             conn_dir->offloads.flags &= ~CT_OFFLOAD_INIT;
         }
-        conn_dir->offloads.flags |= CT_OFFLOAD_DELETED;
     }
 
     offload_class->conn_del(ct, conn);
     conntrack_stats_inc(ct, CT_DPIF_CONN_STATE_OFFLOAD_DEL_REQUESTED);
 mark_deleted:
-    conn_master(conn)->offloads.flags |= CT_OFFLOAD_DELETED;
+    if (flush) {
+        conn_master(conn)->offloads.flags |= CT_OFFLOAD_BYPASS_TCP_SEQ_CHK;
+    } else {
+        conn_master(conn)->offloads.flags |= CT_OFFLOAD_DELETED;
+    }
 }
 
 static void
@@ -329,7 +332,7 @@ conntrack_offload_active(struct conntrack *ct,
     coh = conntrack_offload_get(conn);
     flags = conn->offloads.flags;
 
-    if (flags & (CT_OFFLOAD_DELETED | CT_OFFLOAD_TERMINATED)) {
+    if (flags & CT_OFFLOAD_DELETED) {
         *ret = ENODATA;
     } else if (!ct_offload_handle_active(coh, CT_OFFLOAD_BOTH)) {
         *ret = ENODATA;
