@@ -790,12 +790,14 @@ action_mask_to_changeable_value(struct ovs_doca_flow_actions *ovalue,
 }
 
 static void
-mask_to_changeable_value_path_selector(struct ds *ds, char *prefix,
-                                       struct ovs_doca_flow_match *value,
-                                       struct ovs_doca_flow_match *mask,
-                                       struct ovs_doca_flow_match *used)
+mask_to_changeable_value_nv_mp(struct ds *ds, char *prefix,
+                               struct ovs_doca_flow_match *value,
+                               struct ovs_doca_flow_match *mask,
+                               struct ovs_doca_flow_match *used)
 {
-    extend_field_to_changeable(path_selector);
+    extend_field_to_changeable(nv_mp.pid);
+    extend_field_to_changeable(nv_mp.preferred);
+    extend_field_to_changeable(nv_mp.strict);
 }
 
 static void
@@ -818,7 +820,7 @@ match_mask_to_changeable_value(struct ovs_doca_flow_match *ovalue,
                                            &used->inner, DOCA_FLOW_TUN_NONE);
     mask_to_changeable_value_header_format(ds, "match.outer", &value->outer, &mask->outer,
                                            &used->outer, DOCA_FLOW_TUN_NONE);
-    mask_to_changeable_value_path_selector(ds, "path_selector", ovalue, omask, &u);
+    mask_to_changeable_value_nv_mp(ds, "nv_mp", ovalue, omask, &u);
 
     ovs_assert(is_all_zeros(&u, sizeof u));
 }
