@@ -690,7 +690,7 @@ netdev_doca_set_config_port_dir(const char *devargs,
     }
 }
 
-static int
+int
 get_sys(const char *prefix, const char *devname, const char *suffix,
         char *outp, size_t maxlen)
 {
@@ -1085,7 +1085,7 @@ static int
 get_first_dir_entry(const char *dir, char *peer, size_t maxlen)
 {
     struct dirent *de;
-    int rv = 0;
+    int rv = -1;
     DIR *dirp;
     int n;
 
@@ -1099,10 +1099,10 @@ get_first_dir_entry(const char *dir, char *peer, size_t maxlen)
 
         n = strlen(de->d_name);
         if (n >= maxlen) {
-            rv = -1;
             break;
         }
         ovs_strlcpy(peer, de->d_name, maxlen);
+        rv = 0;
         break;
     }
     closedir(dirp);
@@ -1216,63 +1216,6 @@ get_rep_peer(const char name[], char *peer, size_t maxlen)
     return -1;
 }
 
-struct get_rep_aux {
-    const char *dev;
-    char *rep;
-    size_t maxlen;
-    int rv;
-};
-
-static bool
-get_rep_cb(struct netdev *netdev,
-           odp_port_t odp_port OVS_UNUSED,
-           void *aux_)
-{
-    const char *name = netdev_get_name(netdev);
-    struct get_rep_aux *aux = aux_;
-    char peer_[IFNAMSIZ], *peer;
-    bool allocated = false;
-    bool rv = false;
-
-    if (sizeof peer_ < aux->maxlen) {
-        peer = xmalloc(aux->maxlen);
-        allocated = true;
-    } else {
-        peer = peer_;
-    }
-
-    if (get_rep_peer(name, peer, aux->maxlen)) {
-        goto out;
-    }
-    if (strncmp(aux->dev, peer, aux->maxlen)) {
-        goto out;
-    }
-
-    ovs_strlcpy(aux->rep, name, aux->maxlen);
-    aux->rv = 0;
-    rv = true;
-
-out:
-    if (allocated) {
-        free(peer);
-    }
-    return rv;
-}
-
-int
-netdev_doca_get_rep_name(const char dev[], char *rep, size_t maxlen)
-{
-    struct get_rep_aux aux = {
-        .dev = dev,
-        .rep = rep,
-        .maxlen = maxlen,
-        .rv = -1,
-    };
-
-    netdev_ports_traverse("doca", get_rep_cb, &aux);
-    return aux.rv;
-}
-
 static bool
 netdev_doca_can_use_port(const char *name)
 {
