@@ -998,8 +998,25 @@ ovs_doca_init_defs(struct doca_flow_cfg *cfg,
                    struct doca_flow_definitions **defs,
                    struct doca_flow_definitions_cfg **defs_cfg)
 {
+#define DEF_FIELD(str_val, struct_name, field_name) {     \
+    .str = str_val,                                       \
+    .offset = offsetof(struct struct_name, field_name),   \
+    .size = MEMBER_SIZEOF(struct struct_name, field_name) \
+}
+    struct def_field {
+        const char *str;
+        size_t offset;
+        size_t size;
+    } def_fields[] = {
+        DEF_FIELD("actions.packet.meta.mark", ovs_doca_flow_actions, mark),
+        DEF_FIELD("actions.packet.meta.plb_port_id", ovs_doca_flow_actions, nv_mp.pid),
+        DEF_FIELD("actions.packet.meta.plb_port_strict", ovs_doca_flow_actions, nv_mp.strict),
+        DEF_FIELD("actions.packet.meta.plb_port_preferred", ovs_doca_flow_actions, nv_mp.preferred),
+        DEF_FIELD("match.packet.meta.plb_port_id", ovs_doca_flow_match, nv_mp.pid),
+        DEF_FIELD("match.packet.meta.plb_port_strict", ovs_doca_flow_match, nv_mp.strict),
+        DEF_FIELD("match.packet.meta.plb_port_preferred", ovs_doca_flow_match, nv_mp.preferred),
+    };
     doca_error_t result;
-    const char *def_str;
 
     result = doca_flow_definitions_cfg_create(defs_cfg);
     if (result != DOCA_SUCCESS) {
@@ -1012,34 +1029,14 @@ ovs_doca_init_defs(struct doca_flow_cfg *cfg,
         goto out;
     }
 
-    def_str = "actions.packet.meta.mark";
-    result = doca_flow_definitions_add_field(*defs, def_str,
-                                             offsetof(struct ovs_doca_flow_actions, mark),
-                                             MEMBER_SIZEOF(struct ovs_doca_flow_actions, mark));
-    if (result != DOCA_SUCCESS) {
-        VLOG_ERR("definitions add field '%s' error: result=%d(%s)", def_str,
-                 result, doca_error_get_name(result));
-        goto out;
-    }
-    def_str = "actions.packet.meta.path_selector";
-    result = doca_flow_definitions_add_field(*defs, def_str,
-                                             offsetof(struct ovs_doca_flow_actions, path_selector),
-                                             MEMBER_SIZEOF(struct ovs_doca_flow_actions,
-                                                           path_selector));
-    if (result != DOCA_SUCCESS) {
-        VLOG_ERR("definitions add field '%s' error: result=%d(%s)", def_str,
-                  result, doca_error_get_name(result));
-         goto out;
-    }
-    def_str = "match.packet.meta.path_selector";
-    result = doca_flow_definitions_add_field(*defs, def_str,
-                                             offsetof(struct ovs_doca_flow_match, path_selector),
-                                             MEMBER_SIZEOF(struct ovs_doca_flow_match,
-                                                           path_selector));
-    if (result != DOCA_SUCCESS) {
-        VLOG_ERR("definitions add field '%s' error: result=%d(%s)", def_str,
-                 result, doca_error_get_name(result));
-        goto out;
+    for (int i = 0; i < ARRAY_SIZE(def_fields); i++) {
+        result = doca_flow_definitions_add_field(*defs, def_fields[i].str, def_fields[i].offset,
+                                                 def_fields[i].size);
+        if (result != DOCA_SUCCESS) {
+            VLOG_ERR("definitions add field '%s' error: result=%d(%s)", def_fields[i].str,
+                     result, doca_error_get_name(result));
+            goto out;
+        }
     }
 
     result = doca_flow_cfg_set_definitions(cfg, *defs);
