@@ -1089,8 +1089,7 @@ ipv6_addr_equals_masked(const struct in6_addr *a, const struct in6_addr *b,
     struct in6_addr ma;
     struct in6_addr mb;
 
-    if (((IN6_IS_ADDR_V4MAPPED(a) || IN6_IS_ADDR_V4MAPPED(b)) && plen == 32) ||
-        plen == 128) {
+    if (plen == 128) {
         return ipv6_addr_equals(a, b);
     }
 
