@@ -281,7 +281,7 @@ rule_handle_msg(const struct route_table_msg *change)
         route_table_dump_one_table(rd->lookup_table, route_table_handle_msg,
                                    NULL);
         ovs_router_rule_add(rd->prio, rd->invert, false, rd->src_len,
-                            &rd->from_addr, rd->lookup_table);
+                            &rd->from_addr, rd->lookup_table, rd->ipv4);
     }
 }
 
@@ -356,6 +356,7 @@ rule_parse(struct ofpbuf *buf, void *change_)
         change->rud.invert = false;
         change->rud.src_len = frh->src_len;
         change->rud.lookup_table = frh->table;
+        change->rud.ipv4 = ipv4;
 
         if (frh->flags & FIB_RULE_INVERT) {
             /* Invert the matching of rule selector. */
@@ -376,6 +377,8 @@ rule_parse(struct ofpbuf *buf, void *change_)
         } else {
             if (ipv4) {
                 in6_addr_set_mapped_ipv4(&change->rud.from_addr, 0);
+            } else {
+                change->rud.from_addr = in6addr_any;
             }
             from_addr_any = true;
         }
@@ -402,6 +405,14 @@ rule_parse(struct ofpbuf *buf, void *change_)
         change->relevant = false;
     }
 
+    /* Skip non-standard rules when they are disabled in other_config.
+     */
+    if (!ovs_router_rules_enabled() &&
+        !((change->rud.prio == 0 || change->rud.prio == 0x7FFE || change->rud.prio == 0x7FFF) &&
+          from_addr_any && is_standard_table_id(change->rud.lookup_table))) {
+        change->relevant = false;
+    }
+
     /* Check if there are any additional attributes that aren't supported
      * currently by OVS rule-based route lookup. */
     if (change->relevant) {
