@@ -625,7 +625,7 @@ if [ $missing_pkgs -eq 1 ]; then
 fi
 
 pass_echo "Building $PACKAGE $BINDIR . Please wait..."
-ex ${ofed}/install.pl --tmpdir $tmpdir_logs --kernel-only --kernel $kernel --kernel-sources $kernel_sources --builddir $tmpdir $KMP $force $disabled_pkgs --build-only $distro1 $KMP_BUMP_VER $MLNX_EXTRA_FLAGS
+ex ${ofed}/install.pl --tmpdir $tmpdir_logs --kernel-only --fwctl --kernel $kernel --kernel-sources $kernel_sources --builddir $tmpdir $KMP $force $disabled_pkgs --build-only $distro1 $KMP_BUMP_VER $MLNX_EXTRA_FLAGS
 
 # build mlnx-en
 WITH_ETH_ONLY=0
@@ -658,6 +658,7 @@ bins=`find ${ofed}/$BINDIR \
 	-o -name "*mlnx-nfsrdma*" \
 	-o -name "*mlnx-nvme*" \
 	-o -name "*mlnx-rdma-rxe*" \
+	-o -name "*fwctl*" \
 `
 
 for p in $bins
