@@ -242,28 +242,31 @@ int check_address_space_support(struct nnt_device* nnt_device)
         return error;
     }
 
-    /* Is ICMD address space supported ?*/
-    if (set_address_space(nnt_device, NNT_SPACE_ALL_ICMD) == 0)
+    if (!is_cpu_pci_device(nnt_device->pci_device->device))
     {
-        nnt_device->pciconf_device.address_space.icmd = 1;
-    }
+        /* Is ICMD address space supported ?*/
+        if (set_address_space(nnt_device, NNT_SPACE_ALL_ICMD) == 0)
+        {
+            nnt_device->pciconf_device.address_space.icmd = 1;
+        }
 
-    /* Is CR Space address space supported ?*/
-    if (set_address_space(nnt_device, NNT_SPACE_CR_SPACE) == 0)
-    {
-        nnt_device->pciconf_device.address_space.cr_space = 1;
-    }
+        /* Is CR Space address space supported ?*/
+        if (set_address_space(nnt_device, NNT_SPACE_CR_SPACE) == 0)
+        {
+            nnt_device->pciconf_device.address_space.cr_space = 1;
+        }
 
-    /* Is semaphore address space supported ?*/
-    if (set_address_space(nnt_device, NNT_SPACE_GLOBAL_SEMAPHORE) == 0)
-    {
-        nnt_device->pciconf_device.address_space.semaphore = 1;
-    }
+        /* Is semaphore address space supported ?*/
+        if (set_address_space(nnt_device, NNT_SPACE_GLOBAL_SEMAPHORE) == 0)
+        {
+            nnt_device->pciconf_device.address_space.semaphore = 1;
+        }
 
-    /* Is recovery address space supported ?*/
-    if (set_address_space(nnt_device, NNT_SPACE_RECOVERY) == 0)
-    {
-        nnt_device->pciconf_device.address_space.recovery = 1;
+        /* Is recovery address space supported ?*/
+        if (set_address_space(nnt_device, NNT_SPACE_RECOVERY) == 0)
+        {
+            nnt_device->pciconf_device.address_space.recovery = 1;
+        }
     }
 
     /* Is PCI ICMD address space supported ?*/
