@@ -393,7 +393,7 @@ static void srp_reconnect_work(struct work_struct *work)
 			     "reconnect attempt %d failed (%d)\n",
 			     ++rport->failed_reconnects, res);
 		delay = rport->reconnect_delay *
-			min(100, max(1, rport->failed_reconnects - 10));
+			clamp(rport->failed_reconnects - 10, 1, 100);
 		if (delay > 0)
 			queue_delayed_work(system_long_wq,
 					   &rport->reconnect_work, delay * HZ);
@@ -401,7 +401,7 @@ static void srp_reconnect_work(struct work_struct *work)
 }
 
 /*
- * scsi_target_block() must have been called before this function is
+ * scsi_block_targets() must have been called before this function is
  * called to guarantee that no .queuecommand() calls are in progress.
  */
 static void __rport_fail_io_fast(struct srp_rport *rport)
@@ -577,21 +577,12 @@ int srp_reconnect_rport(struct srp_rport *rport)
 		 * invoking scsi_target_unblock() won't change the state of
 		 * these devices into running so do that explicitly.
 		 */
-#ifdef HAVE_SCSI_DEVICE_STATE_MUTEX
 		shost_for_each_device(sdev, shost) {
 			mutex_lock(&sdev->state_mutex);
-#else
-		spin_lock_irq(shost->host_lock);
-		__shost_for_each_device(sdev, shost)
-#endif
 			if (sdev->sdev_state == SDEV_OFFLINE)
 				sdev->sdev_state = SDEV_RUNNING;
-#ifdef HAVE_SCSI_DEVICE_STATE_MUTEX
 			mutex_unlock(&sdev->state_mutex);
 		}
-#else
-		spin_unlock_irq(shost->host_lock);
-#endif
 	} else if (rport->state == SRP_RPORT_RUNNING) {
 		/*
 		 * srp_reconnect_rport() has been invoked with fast_io_fail
@@ -640,11 +631,7 @@ enum blk_eh_timer_return srp_timed_out(struct scsi_cmnd *scmd)
 #ifdef HAVE_SCSI_TIMEOUT_ACTION
 		SCSI_EH_RESET_TIMER : SCSI_EH_NOT_HANDLED;
 #else
-#ifdef HAVE_BLK_EH_DONE
 		BLK_EH_RESET_TIMER : BLK_EH_DONE;
-#else
-		BLK_EH_RESET_TIMER : BLK_EH_NOT_HANDLED;
-#endif
 #endif
 }
 EXPORT_SYMBOL(srp_timed_out);
