@@ -86,8 +86,6 @@ static inline uint32_t get_mlx5_opcode(xlio_ibv_wr_opcode verbs_opcode)
         return MLX5_OPCODE_SEND;
     case XLIO_IBV_WR_TSO:
         return MLX5_OPCODE_TSO;
-    case XLIO_IBV_WR_NOP:
-        return MLX5_OPCODE_NOP;
     default:
         return MLX5_OPCODE_SEND;
     }
@@ -111,9 +109,6 @@ hw_queue_tx::hw_queue_tx(ring_simple *ring, const slave_data_t *slave, const uin
 
     memset(&m_rate_limit, 0, sizeof(struct xlio_rate_limit_t));
 
-    // Check device capabilities for dummy send support
-    m_hw_dummy_send_support = xlio_is_nop_supported(m_p_ib_ctx_handler->get_ibv_device_attr());
-
     if (configure(slave)) {
         throw_xlio_exception("Failed to configure");
     }
@@ -163,7 +158,6 @@ int hw_queue_tx::configure(const slave_data_t *slave)
                  priv_xlio_transport_type_str(m_p_ring->get_transport_type()),
                  m_p_ib_ctx_handler->get_ibname(), m_p_ib_ctx_handler->get_ibv_device(),
                  m_port_num);
-    hwqtx_logdbg("HW Dummy send support for QP = %d", m_hw_dummy_send_support);
 
     // Create associated cq_mgr_tx and unused cq_mgr_rx_regrq just for QP sake.
     BULLSEYE_EXCLUDE_BLOCK_START
@@ -309,15 +303,13 @@ void hw_queue_tx::down()
 
 void hw_queue_tx::release_tx_buffers()
 {
-    int ret;
     uint64_t poll_sn = 0;
     hwqtx_logdbg("draining cq_mgr_tx %p", m_p_cq_mgr_tx);
     while (m_p_cq_mgr_tx && m_mlx5_qp.qp &&
-           ((ret = m_p_cq_mgr_tx->poll_and_process_element_tx(&poll_sn)) > 0) &&
+           (m_p_cq_mgr_tx->poll_and_process_element_tx(&poll_sn) > 0) &&
            (errno != EIO && !m_p_ib_ctx_handler->is_removed())) {
-        hwqtx_logdbg("draining completed on cq_mgr_tx (%d wce)", ret);
+        hwqtx_logdbg("draining completed on cq_mgr_tx");
     }
-    NOT_IN_USE(ret); // Suppress --enable-opt-log=high warning
 }
 
 void hw_queue_tx::send_wqe(xlio_ibv_send_wr *p_send_wqe, xlio_wr_tx_packet_attr attr, xlio_tis *tis,
@@ -342,13 +334,8 @@ void hw_queue_tx::send_wqe(xlio_ibv_send_wr *p_send_wqe, xlio_wr_tx_packet_attr
 
     if (!skip_tx_poll && is_signal_requested_for_last_wqe()) {
         uint64_t dummy_poll_sn = 0;
-        int ret = m_p_cq_mgr_tx->poll_and_process_element_tx(&dummy_poll_sn);
-        BULLSEYE_EXCLUDE_BLOCK_START
-        if (ret < 0) {
-            hwqtx_logerr("error from cq_mgr_tx->process_next_element (ret=%d %m)", ret);
-        }
-        BULLSEYE_EXCLUDE_BLOCK_END
-        hwqtx_logfunc("polling succeeded on cq_mgr_tx (%d wce)", ret);
+        m_p_cq_mgr_tx->poll_and_process_element_tx(&dummy_poll_sn);
+        hwqtx_logfunc("polling succeeded on cq_mgr_tx");
     }
 }
 
@@ -460,13 +447,6 @@ void hw_queue_tx::init_queue()
     hwqtx_logfunc("m_tx_num_wr=%d max_inline_data: %d m_sq_wqe_idx_to_prop=%p", m_tx_num_wr,
                   get_max_inline_data(), m_sq_wqe_idx_to_prop);
 
-    memset((void *)(uintptr_t)m_sq_wqe_hot, 0, sizeof(struct mlx5_eth_wqe));
-    m_sq_wqe_hot->ctrl.data[0] = htonl(MLX5_OPCODE_SEND);
-    m_sq_wqe_hot->ctrl.data[1] = htonl((m_mlx5_qp.qpn << 8) | 4);
-    m_sq_wqe_hot->ctrl.data[2] = 0;
-    m_sq_wqe_hot->eseg.inline_hdr_sz = htons(MLX5_ETH_INLINE_HEADER_SIZE);
-    m_sq_wqe_hot->eseg.cs_flags = XLIO_TX_PACKET_L3_CSUM | XLIO_TX_PACKET_L4_CSUM;
-
     hwqtx_logfunc("%p allocated for %d QPs sq_wqes:%p sq_wqes_end: %p and configured %d WRs "
                   "BlueFlame: %p",
                   m_mlx5_qp.qp, m_mlx5_qp.qpn, m_sq_wqes, m_sq_wqes_end, m_tx_num_wr,
@@ -487,15 +467,9 @@ void hw_queue_tx::init_device_memory()
 
 void hw_queue_tx::update_next_wqe_hot()
 {
-    // Preparing next WQE as Ethernet send WQE and index:
+    // Preparing pointer to the next WQE after a doorbell
     m_sq_wqe_hot = &(*m_sq_wqes)[m_sq_wqe_counter & (m_tx_num_wr - 1)];
     m_sq_wqe_hot_index = m_sq_wqe_counter & (m_tx_num_wr - 1);
-    memset(m_sq_wqe_hot, 0, sizeof(mlx5_eth_wqe));
-
-    // Fill Ethernet segment with header inline:
-    struct mlx5_wqe_eth_seg *eth_seg =
-        (struct mlx5_wqe_eth_seg *)((uint8_t *)m_sq_wqe_hot + sizeof(struct mlx5_wqe_ctrl_seg));
-    eth_seg->inline_hdr_sz = htons(MLX5_ETH_INLINE_HEADER_SIZE);
 }
 
 cq_mgr_tx *hw_queue_tx::init_tx_cq_mgr()
@@ -582,6 +556,10 @@ inline int hw_queue_tx::fill_wqe(xlio_ibv_send_wr *pswr)
             "wqe_hot:%p num_sge: %d data_addr: %p data_len: %d max_inline_len: %d inline_len: %d",
             m_sq_wqe_hot, pswr->num_sge, data_addr, data_len, max_inline_len, inline_len);
 
+        struct mlx5_wqe_eth_seg *eth_seg =
+            (struct mlx5_wqe_eth_seg *)((uint8_t *)m_sq_wqe_hot + sizeof(struct mlx5_wqe_ctrl_seg));
+        eth_seg->inline_hdr_sz = htons(MLX5_ETH_INLINE_HEADER_SIZE);
+
         // Fill Ethernet segment with header inline, static data
         // were populated in preset after previous packet send
         memcpy(cur_seg + offsetof(struct mlx5_wqe_eth_seg, inline_hdr_start), data_addr,
@@ -823,6 +801,8 @@ void hw_queue_tx::send_to_wire(xlio_ibv_send_wr *p_send_wqe, xlio_wr_tx_packet_a
     ctrl = (struct xlio_mlx5_wqe_ctrl_seg *)m_sq_wqe_hot;
     eseg = (struct mlx5_wqe_eth_seg *)((uint8_t *)m_sq_wqe_hot + sizeof(*ctrl));
 
+    memset(m_sq_wqe_hot, 0, sizeof(mlx5_eth_wqe));
+
     /* Configure ctrl segment
      * qpn_ds or ctrl.data[1] is set inside fill_wqe()
      */
@@ -874,7 +854,7 @@ std::unique_ptr<xlio_tis> hw_queue_tx::create_tis(uint32_t flags)
 
     dpcp::tis *dpcp_tis = nullptr;
     if (unlikely(adapter->create_tis(tis_attr, dpcp_tis) != dpcp::DPCP_OK)) {
-        hwqtx_logerr("Failed to create TIS with NVME enabled");
+        hwqtx_logerr("Failed to create TIS");
         return nullptr;
     }
 
@@ -882,99 +862,6 @@ std::unique_ptr<xlio_tis> hw_queue_tx::create_tis(uint32_t flags)
     return std::make_unique<xlio_tis>(this, std::unique_ptr<dpcp::tis>(dpcp_tis), tis_type);
 }
 
-static inline void nvme_fill_static_params_control(xlio_mlx5_wqe_ctrl_seg *cseg,
-                                                   xlio_mlx5_wqe_umr_ctrl_seg *ucseg,
-                                                   uint32_t producer_index, uint32_t qpn,
-                                                   uint32_t tisn, uint8_t fence_flags)
-{
-    memset(cseg, 0, sizeof(*cseg));
-    memset(ucseg, 0, sizeof(*ucseg));
-    cseg->opmod_idx_opcode =
-        htobe32(((producer_index & 0xffff) << 8) | MLX5_OPCODE_UMR |
-                (MLX5_CTRL_SEGMENT_OPC_MOD_UMR_NVMEOTCP_TIS_STATIC_PARAMS << 24));
-    size_t num_wqe_ds = 12U;
-    cseg->qpn_ds = htobe32((qpn << MLX5_WQE_CTRL_QPN_SHIFT) | num_wqe_ds);
-    cseg->fm_ce_se = fence_flags;
-    cseg->tis_tir_num = htobe32(tisn << MLX5_WQE_CTRL_TIR_TIS_INDEX_SHIFT);
-
-    ucseg->flags = MLX5_UMR_INLINE;
-    ucseg->bsf_octowords = htobe16(MLX5E_TRANSPORT_STATIC_PARAMS_OCTWORD_SIZE);
-}
-
-static inline void nvme_fill_static_params_transport_params(
-    mlx5_wqe_transport_static_params_seg *params, uint32_t config)
-
-{
-    memset(params, 0, sizeof(*params));
-    void *ctx = params->ctx;
-
-    DEVX_SET(transport_static_params, ctx, const_1, 1);
-    DEVX_SET(transport_static_params, ctx, const_2, 2);
-    DEVX_SET(transport_static_params, ctx, acc_type, MLX5_TRANSPORT_STATIC_PARAMS_ACC_TYPE_NVMETCP);
-    DEVX_SET(transport_static_params, ctx, nvme_resync_tcp_sn, 0);
-    DEVX_SET(transport_static_params, ctx, pda, static_cast<uint8_t>(config & XLIO_NVME_PDA_MASK));
-    DEVX_SET(transport_static_params, ctx, ddgst_en, bool(config & XLIO_NVME_DDGST_ENABLE));
-    DEVX_SET(transport_static_params, ctx, ddgst_offload_en,
-             bool(config & XLIO_NVME_DDGST_OFFLOAD));
-    DEVX_SET(transport_static_params, ctx, hddgst_en, bool(config & XLIO_NVME_HDGST_ENABLE));
-    DEVX_SET(transport_static_params, ctx, hdgst_offload_en,
-             bool(config & XLIO_NVME_HDGST_OFFLOAD));
-    DEVX_SET(transport_static_params, ctx, ti, MLX5_TRANSPORT_STATIC_PARAMS_TI_INITIATOR);
-    DEVX_SET(transport_static_params, ctx, const1, 1);
-    DEVX_SET(transport_static_params, ctx, zero_copy_en, 0);
-}
-
-static inline void nvme_fill_progress_wqe(mlx5e_set_nvmeotcp_progress_params_wqe *wqe,
-                                          uint32_t producer_index, uint32_t qpn, uint32_t tisn,
-                                          uint32_t tcp_seqno, uint8_t fence_flags)
-{
-    memset(wqe, 0, sizeof(*wqe));
-    auto cseg = &wqe->ctrl.ctrl;
-
-    size_t progres_params_ds = DIV_ROUND_UP(sizeof(*wqe), MLX5_SEND_WQE_DS);
-    cseg->opmod_idx_opcode =
-        htobe32(((producer_index & 0xffff) << 8) | XLIO_MLX5_OPCODE_SET_PSV |
-                (MLX5_CTRL_SEGMENT_OPC_MOD_UMR_NVMEOTCP_TIS_PROGRESS_PARAMS << 24));
-    cseg->qpn_ds = htobe32((qpn << MLX5_WQE_CTRL_QPN_SHIFT) | progres_params_ds);
-    cseg->fm_ce_se = fence_flags;
-
-    mlx5_seg_nvmeotcp_progress_params *params = &wqe->params;
-    params->tir_num = htobe32(tisn);
-    void *ctx = params->ctx;
-
-    DEVX_SET(nvmeotcp_progress_params, ctx, next_pdu_tcp_sn, tcp_seqno);
-    DEVX_SET(nvmeotcp_progress_params, ctx, pdu_tracker_state,
-             MLX5E_NVMEOTCP_PROGRESS_PARAMS_PDU_TRACKER_STATE_START);
-    /* if (is_tx) offloading state == 0*/
-    DEVX_SET(nvmeotcp_progress_params, ctx, offloading_state, 0);
-}
-
-void hw_queue_tx::nvme_set_static_context(xlio_tis *tis, uint32_t config)
-{
-    auto *cseg = wqebb_get<xlio_mlx5_wqe_ctrl_seg *>(0U);
-    auto *ucseg = wqebb_get<xlio_mlx5_wqe_umr_ctrl_seg *>(0U, sizeof(*cseg));
-
-    nvme_fill_static_params_control(cseg, ucseg, m_sq_wqe_counter, m_mlx5_qp.qpn, tis->get_tisn(),
-                                    0);
-    memset(wqebb_get<void *>(1U), 0, sizeof(mlx5_mkey_seg));
-
-    auto *params = wqebb_get<mlx5_wqe_transport_static_params_seg *>(2U);
-    nvme_fill_static_params_transport_params(params, config);
-    store_current_wqe_prop(nullptr, SQ_CREDITS_UMR, tis);
-    ring_doorbell(MLX5E_TRANSPORT_SET_STATIC_PARAMS_WQEBBS);
-    update_next_wqe_hot();
-}
-
-void hw_queue_tx::nvme_set_progress_context(xlio_tis *tis, uint32_t tcp_seqno)
-{
-    auto *wqe = reinterpret_cast<mlx5e_set_nvmeotcp_progress_params_wqe *>(m_sq_wqe_hot);
-    nvme_fill_progress_wqe(wqe, m_sq_wqe_counter, m_mlx5_qp.qpn, tis->get_tisn(), tcp_seqno,
-                           MLX5_FENCE_MODE_INITIATOR_SMALL);
-    store_current_wqe_prop(nullptr, SQ_CREDITS_SET_PSV, tis);
-    ring_doorbell(MLX5E_NVMEOTCP_PROGRESS_PARAMS_WQEBBS);
-    update_next_wqe_hot();
-}
-
 #if defined(DEFINED_UTLS)
 std::unique_ptr<dpcp::tls_dek> hw_queue_tx::get_new_tls_dek(const void *key,
                                                             uint32_t key_size_bytes)
@@ -1233,7 +1120,6 @@ inline void hw_queue_tx::tls_post_static_params_wqe(xlio_ti *ti, const struct xl
      * one WQEBB and will be posted before m_sq_wqes_end.
      */
 
-    // XXX: We set inline_hdr_sz for every new hot wqe. This corrupts UMR WQE without memset().
     memset(m_sq_wqe_hot, 0, sizeof(*m_sq_wqe_hot));
     cseg->opmod_idx_opcode =
         htobe32(((m_sq_wqe_counter & 0xffff) << 8) | MLX5_OPCODE_UMR | (opmod << 24));
@@ -1297,7 +1183,7 @@ inline void hw_queue_tx::tls_post_progress_params_wqe(xlio_ti *ti, uint32_t tis_
     uint8_t opmod =
         is_tx ? MLX5_OPC_MOD_TLS_TIS_PROGRESS_PARAMS : MLX5_OPC_MOD_TLS_TIR_PROGRESS_PARAMS;
 
-    memset(wqe, 0, sizeof(*wqe));
+    memset(wqe, 0, std::max<size_t>(WQEBB, sizeof(mlx5_set_tls_progress_params_wqe)));
 
 #define PROGRESS_PARAMS_DS_CNT DIV_ROUND_UP(sizeof(*wqe), MLX5_SEND_WQE_DS)
 
@@ -1326,7 +1212,7 @@ inline void hw_queue_tx::tls_get_progress_params_wqe(xlio_ti *ti, uint32_t tirn,
     struct xlio_mlx5_seg_get_psv *psv = &wqe->psv;
     uint8_t opmod = MLX5_OPC_MOD_TLS_TIR_PROGRESS_PARAMS;
 
-    memset(wqe, 0, sizeof(*wqe));
+    memset(wqe, 0, std::max<size_t>(WQEBB, sizeof(mlx5_get_tls_progress_params_wqe)));
 
 #define PROGRESS_PARAMS_DS_CNT DIV_ROUND_UP(sizeof(*wqe), MLX5_SEND_WQE_DS)
 
@@ -1398,7 +1284,7 @@ void hw_queue_tx::post_nop_fence(void)
     struct mlx5_wqe *wqe = reinterpret_cast<struct mlx5_wqe *>(m_sq_wqe_hot);
     struct xlio_mlx5_wqe_ctrl_seg *cseg = &wqe->ctrl;
 
-    memset(wqe, 0, sizeof(*wqe));
+    memset(wqe, 0, std::max<size_t>(WQEBB, sizeof(mlx5_wqe)));
 
     cseg->opmod_idx_opcode = htobe32(((m_sq_wqe_counter & 0xffff) << 8) | MLX5_OPCODE_NOP);
     cseg->qpn_ds = htobe32((m_mlx5_qp.qpn << MLX5_WQE_CTRL_QPN_SHIFT) | 0x01);
@@ -1420,7 +1306,7 @@ void hw_queue_tx::post_dump_wqe(xlio_tis *tis, void *addr, uint32_t len, uint32_
     uint32_t tisn = tis ? tis->get_tisn() : 0;
     uint16_t ds_cnt = sizeof(*wqe) / MLX5_SEND_WQE_DS;
 
-    memset(wqe, 0, sizeof(*wqe));
+    memset(wqe, 0, std::max<size_t>(WQEBB, sizeof(mlx5_dump_wqe)));
 
     cseg->opmod_idx_opcode = htobe32(((m_sq_wqe_counter & 0xffff) << 8) | XLIO_MLX5_OPCODE_DUMP);
     cseg->qpn_ds = htobe32((m_mlx5_qp.qpn << MLX5_WQE_CTRL_QPN_SHIFT) | ds_cnt);
