@@ -417,8 +417,7 @@ int io_mux_call::call()
 
     __log_funcall("");
 
-    if (!safe_mce_sys().select_poll_os_force // TODO: evaluate/consider this logic
-        && (*m_p_num_all_offloaded_fds == 0)) {
+    if (*m_p_num_all_offloaded_fds == 0) {
         // 1st scenario
         timer_update();
         wait_os(false);
@@ -433,7 +432,8 @@ int io_mux_call::call()
             check_all_offloaded_sockets();
             if (m_n_all_ready_fds) {
                 goto done;
-            } else { // false wake-up, and we already discovered that we should be in 2nd scenario
+            } else { // false wake-up, and we already discovered that we should be in 2nd
+                     // scenario
                 timer_update();
                 if (is_timeout(m_elapsed)) {
                     goto done;
@@ -494,20 +494,31 @@ bool io_mux_call::immidiate_return(int &poll_os_countdown)
 
 bool io_mux_call::ring_poll_and_process_element()
 {
-    // TODO: (select, poll) this access all CQs, it is better to check only relevant ones
-    return g_p_net_device_table_mgr->global_ring_poll_and_process_element(&m_poll_sn_rx,
-                                                                          &m_poll_sn_tx, nullptr);
+    if (!safe_mce_sys().is_threads_mode()) {
+        // TODO: (select, poll) this access all CQs, it is better to check only relevant ones
+        return g_p_net_device_table_mgr->global_ring_poll_and_process_element(
+            &m_poll_sn_rx, &m_poll_sn_tx, nullptr);
+    }
+
+    return true;
 }
 
 int io_mux_call::ring_request_notification()
 {
-    return g_p_net_device_table_mgr->global_ring_request_notification(m_poll_sn_rx, m_poll_sn_tx);
+    if (!safe_mce_sys().is_threads_mode()) {
+        return g_p_net_device_table_mgr->global_ring_request_notification(m_poll_sn_rx,
+                                                                          m_poll_sn_tx);
+    }
+
+    return 0;
 }
 
 void io_mux_call::ring_wait_for_notification_and_process_element(void *pv_fd_ready_array)
 {
-    g_p_net_device_table_mgr->global_ring_wait_for_notification_and_process_element(
-        &m_poll_sn_rx, pv_fd_ready_array);
+    if (!safe_mce_sys().is_threads_mode()) {
+        g_p_net_device_table_mgr->global_ring_wait_for_notification_and_process_element(
+            &m_poll_sn_rx, pv_fd_ready_array);
+    }
 }
 
 bool io_mux_call::is_sig_pending()
