@@ -13,17 +13,20 @@
 #include "dev/buffer_pool.h"
 #include "dev/cq_mgr_rx.h"
 #include "xlio_extra.h"
+#include <atomic>
+#include <vector>
 
 #include "lwip/opt.h"
 #include "lwip/tcp_impl.h"
 
 #include "sockinfo.h"
 #include "sockinfo_ulp.h"
-#include "sockinfo_nvme.h"
+#include "sockinfo_tcp_listen_context.h"
 
 /* Forward declarations */
 struct xlio_socket_attr;
 class poll_group;
+class sockinfo_tcp;
 
 #define BLOCK_THIS_RUN(blocking, flags) (blocking && !(flags & MSG_DONTWAIT))
 
@@ -70,8 +73,9 @@ enum xlio_express_flags : uint32_t {
     XLIO_EXPRESS_OP_TYPE_DESC,
     XLIO_EXPRESS_OP_TYPE_FILE_ZEROCOPY,
     XLIO_EXPRESS_OP_TYPE_MASK = 0x000fu,
-    XLIO_EXPRESS_MSG_MORE,
-    XLIO_EXPRESS_MSG_MASK = 0x00f0u,
+    XLIO_EXPRESS_MSG_MORE = 0x0010u,
+    XLIO_EXPRESS_MSG_SND_BUF = 0x0020u,
+    XLIO_EXPRESS_MSG_MASK = 0x0ff0u,
 };
 
 struct socket_option_t {
@@ -108,8 +112,6 @@ public:
 
     void handle_timer_expired(void *user_data) override;
 
-    void register_wakeup_event();
-
     void add_new_timer(sockinfo_tcp *sock);
 
     void remove_timer(sockinfo_tcp *sock);
@@ -117,9 +119,6 @@ public:
     void set_group(poll_group *group) { m_p_group = group; }
     inline event_handler_manager *get_event_mgr();
 
-private:
-    void free_tta_resources();
-
 protected:
     void *m_timer_handle = nullptr;
 
@@ -188,6 +187,8 @@ public:
     void create_dst_entry();
     void destructor_helper_tcp();
     bool prepare_dst_to_send(bool is_accepted_socket = false);
+    void rx_data_recvd(uint32_t tot_size) override;
+    void clear_rx_ready_buffers();
 
     int fcntl(int __cmd, unsigned long int __arg) override;
     int fcntl64(int __cmd, unsigned long int __arg) override;
@@ -198,6 +199,23 @@ public:
     int getsockopt(int __level, int __optname, void *__optval, socklen_t *__optlen) override;
     int getsockopt_offload(int __level, int __optname, void *__optval, socklen_t *__optlen);
     int connect(const sockaddr *, socklen_t) override;
+    void connect_entity_context() override;
+    void listen_entity_context();
+    int harvest_sockinfo_tcp_listen_objects();
+    inline bool try_harvest_from_rss_child(size_t rss_child_index);
+    void set_entity_context(entity_context *ctx) override;
+
+    // Listen context management
+    void create_listen_context();
+    void destroy_listen_context();
+    bool is_sockinfo_tcp_listen_rss_child();
+    sockinfo_tcp_listen_context *get_listen_context() { return m_listen_ctx; }
+    sockinfo_tcp_listen_context *get_parent_listen_context()
+    {
+        return (m_listen_ctx && m_listen_ctx->get_parent_listen_socket())
+            ? m_listen_ctx->get_parent_listen_socket()->m_listen_ctx
+            : nullptr;
+    }
     int bind(const sockaddr *__addr, socklen_t __addrlen) override;
     int listen(int backlog) override;
     int accept(struct sockaddr *__addr, socklen_t *__addrlen) override;
@@ -210,10 +228,6 @@ public:
     inline bool handle_bind_no_port(int &bind_ret, in_port_t in_port, const sockaddr *__addr,
                                     socklen_t __addrlen);
     inline void non_tcp_recved(int rx_len);
-    int recvfrom_zcopy_free_packets(struct xlio_recvfrom_zcopy_packet_t *pkts,
-                                    size_t count) override;
-
-    void socketxtreme_recv_buffs_tcp(mem_buf_desc_t *desc, uint16_t len);
 
     void statistics_print(vlog_levels_t log_level = VLOG_DEBUG) override;
 
@@ -221,13 +235,16 @@ public:
 
     inline unsigned sndbuf_available()
     {
-        return static_cast<unsigned>(std::max(tcp_sndbuf(&m_pcb), 0));
+        return static_cast<unsigned>(std::max(m_snd_buf.load(), 0));
     }
 
     inline unsigned get_mss() { return m_pcb.mss; }
 
     ssize_t tx(xlio_tx_call_attr_t &tx_arg) override;
     ssize_t tcp_tx(xlio_tx_call_attr_t &tx_arg);
+    ssize_t tcp_tx_thread(xlio_tx_call_attr_t &tx_arg);
+    void tx_thread_commit(mem_buf_desc_t *buf_list, uint32_t offset, uint32_t size,
+                          int flags) override;
     ssize_t rx(const rx_call_t call_type, iovec *p_iov, ssize_t sz_iov, int *p_flags,
                sockaddr *__from = nullptr, socklen_t *__fromlen = nullptr,
                struct msghdr *__msg = nullptr) override;
@@ -255,10 +272,7 @@ public:
     uint32_t get_next_tcp_seqno() { return m_pcb.snd_lbb; }
     uint32_t get_next_tcp_seqno_rx() { return m_pcb.rcv_nxt; }
 
-    mem_buf_desc_t *tcp_tx_zc_alloc(mem_buf_desc_t *p_desc);
     static void tcp_express_zc_callback(mem_buf_desc_t *p_desc);
-    static void tcp_tx_zc_callback(mem_buf_desc_t *p_desc);
-    void tcp_tx_zc_handle(mem_buf_desc_t *p_desc);
 
     bool is_readable(uint64_t *p_poll_sn, fd_array_t *p_fd_array = NULL) override;
     bool is_writeable() override;
@@ -295,9 +309,6 @@ public:
     }
 
     bool is_incoming() override { return m_b_incoming; }
-    bool is_timer_registered() const { return m_timer_registered; }
-    void set_timer_registered(bool v) { m_timer_registered = v; }
-
     bool is_connected() { return m_sock_state == TCP_SOCK_CONNECTED_RDWR; }
 
     inline bool is_rtr()
@@ -322,6 +333,8 @@ public:
     inline fd_type_t get_type() override { return FD_TYPE_SOCKET; }
 
     void handle_timer_expired();
+    bool is_timer_registered() const { return m_timer_registered; }
+    void set_timer_registered(bool v) { m_timer_registered = v; }
 
     inline ib_ctx_handler *get_ctx()
     {
@@ -344,7 +357,7 @@ public:
 
     void rx_poll_on_tx_if_needed()
     {
-        if (m_sysvar_rx_poll_on_tx_tcp) {
+        if (m_sysvar_rx_poll_on_tx_tcp && !m_entity_context) {
             int poll_count = 0;
             rx_wait_helper(poll_count, false);
         }
@@ -363,36 +376,24 @@ public:
 
     bool is_utls_supported(int direction) const;
 
-    int get_supported_nvme_feature_mask() const;
-
     inline int trylock_tcp_con() { return m_tcp_con_lock.trylock(); }
     inline void lock_tcp_con() { m_tcp_con_lock.lock(); }
     inline void unlock_tcp_con() { m_tcp_con_lock.unlock(); }
     tcp_timers_collection *get_tcp_timer_collection();
     bool is_cleaned() const { return m_is_cleaned; }
     static err_t rx_lwip_cb(void *arg, struct tcp_pcb *tpcb, struct pbuf *p, err_t err);
-    static err_t rx_lwip_cb_socketxtreme(void *arg, struct tcp_pcb *tpcb, struct pbuf *p,
-                                         err_t err);
-    static err_t rx_lwip_cb_recv_callback(void *arg, struct tcp_pcb *pcb, struct pbuf *p,
-                                          err_t err);
     static err_t rx_drop_lwip_cb(void *arg, struct tcp_pcb *tpcb, struct pbuf *p, err_t err);
-    inline void rx_lwip_cb_socketxtreme_helper(pbuf *p);
-
-    int register_callback(xlio_recv_callback_t callback, void *context) override
-    {
-        tcp_recv(&m_pcb, sockinfo_tcp::rx_lwip_cb_recv_callback);
-        return register_callback_ctx(callback, context);
-    }
 
     int tcp_tx_express(const struct iovec *iov, unsigned iov_len, uint32_t mkey, unsigned flags,
                        void *opaque_op);
     int tcp_tx_express_inline(const struct iovec *iov, unsigned iov_len, unsigned flags);
     void flush();
-
+    void make_dirty();
     void set_xlio_socket(const struct xlio_socket_attr *attr);
+    int update_xlio_socket(unsigned flags, uintptr_t userdata_sq);
     void add_tx_ring_to_group();
-    bool is_xlio_socket() { return m_p_group != nullptr; }
-    poll_group *get_poll_group() { return m_p_group; }
+    int detach_xlio_group();
+    int attach_xlio_group(poll_group *group);
     void xlio_socket_event(int event, int value);
     static err_t rx_lwip_cb_xlio_socket(void *arg, struct tcp_pcb *tpcb, struct pbuf *p, err_t err);
     static void err_lwip_cb_xlio_socket(void *pcb_container, err_t err);
@@ -413,15 +414,15 @@ private:
     inline void lwip_pbuf_init_custom(mem_buf_desc_t *p_desc);
 
     void tcp_timer();
-
+    bool poll_and_progress_rx(uint64_t &poll_sn);
+    bool check_last_rx_poll_progress(unsigned int prev_sndbuf, bool all_drained);
     bool prepare_listen_to_close();
+    void remove_received_syn_socket(sockinfo_tcp *accepted);
+    void accept_connection_xlio_socket(sockinfo_tcp *new_sock);
 
     // Builds rfs key
     static void create_flow_tuple_key_from_pcb(flow_tuple &key, struct tcp_pcb *pcb);
 
-    // auto accept function
-    static void accept_connection_socketxtreme(sockinfo_tcp *parent, sockinfo_tcp *child);
-
     // accept cb func
     static err_t accept_lwip_cb(void *arg, struct tcp_pcb *child_pcb, err_t err);
 
@@ -438,10 +439,21 @@ private:
     // @param newpcb The new pcb. Can be nullptr.
     static void accepted_pcb_cb(struct tcp_pcb *newpcb);
 
+    static err_t rx_lwip_cb_entity_context(void *arg, struct tcp_pcb *tpcb, struct pbuf *p,
+                                           err_t err);
+
+    static void update_ready_packets_stats(socket_stats_t *stats, uint64_t bytes);
+    void stats_update_tx_errors(int error_number);
+
     int accept_helper(struct sockaddr *__addr, socklen_t *__addrlen, int __flags = 0);
 
     // clone socket in accept call
     sockinfo_tcp *accept_clone();
+
+    bool create_listen_rss_children();
+    void start_sockinfo_tcp_listen_objects();
+    bool wait_for_listen_rss_children_ready();
+
     // connect() helper & callback func
     int wait_for_conn_ready_blocking();
     static err_t connect_lwip_cb(void *arg, struct tcp_pcb *tpcb, err_t err);
@@ -451,17 +463,13 @@ private:
     void handle_incoming_handshake_failure(sockinfo_tcp *child_conn);
 
     // rx
-    // int rx_wait(int &poll_count, bool blocking = true);
-    static err_t ack_recvd_lwip_cb(void *arg, struct tcp_pcb *tpcb, u16_t space);
+    static err_t ack_recvd_lwip_cb(void *arg, struct tcp_pcb *tpcb, u32_t acked);
 
-    ssize_t tcp_tx_handle_done_and_unlock(ssize_t total_tx, int errno_tmp, bool is_dummy,
-                                          bool is_send_zerocopy);
+    ssize_t tcp_tx_handle_done_and_unlock(ssize_t total_tx, int errno_tmp);
     ssize_t tcp_tx_handle_errno_and_unlock(int error_number);
     ssize_t tcp_tx_handle_partial_send_and_unlock(ssize_t total_tx, int errno_to_report,
-                                                  bool is_dummy, bool is_send_zerocopy,
                                                   int errno_to_restore);
-    ssize_t tcp_tx_handle_sndbuf_unavailable(ssize_t total_tx, bool is_dummy, bool is_send_zerocopy,
-                                             int errno_to_restore);
+    ssize_t tcp_tx_handle_sndbuf_unavailable(ssize_t total_tx, int errno_to_restore);
     ssize_t tcp_tx_slow_path(xlio_tx_call_attr_t &tx_arg);
     err_t handle_fin(struct tcp_pcb *pcb, err_t err);
     void handle_rx_lwip_cb_error(pbuf *p);
@@ -473,10 +481,17 @@ private:
     void set_conn_properties_from_pcb();
     void set_sock_options(sockinfo_tcp *new_sock);
     void passthrough_unlock(const char *dbg);
-    // Register to timer
     void register_timer();
-
+    void remove_timer();
     void handle_socket_linger();
+    bool connect_bind_any_and_check_rules();
+    void connect_async_set_errs();
+    void connect_threads_mode();
+    int rx_wait_for_data(int in_flags, struct msghdr *__msg, loops_timer &rcv_timeout);
+    int rx_sleep_wait(loops_timer &rcv_timeout);
+
+    ssize_t rx_read_ready_packets(iovec *p_iov, ssize_t sz_iov, int *p_flags, sockaddr *__from,
+                                  socklen_t *__fromlen, struct msghdr *__msg);
 
     /*
      * Supported only for UDP
@@ -542,8 +557,9 @@ private:
         }
     }
 
-    void post_dequeue(bool release_buff) override;
-    int zero_copy_rx(iovec *p_iov, mem_buf_desc_t *pdesc, int *p_flags) override;
+    void post_dequeue() override {};
+
+    size_t rx_fetch_ready_buffers(iovec *p_iov, iovec *p_iov_end, struct msghdr *__msg);
 
     // Returns the connected pcb, with 5 tuple which matches the input arguments,
     // in state "SYN Received" or NULL if pcb wasn't found
@@ -562,7 +578,8 @@ private:
     inline int rx_wait_lockless(int &poll_count, bool blocking);
     int rx_wait_helper(int &poll_count, bool blocking);
     void fit_rcv_wnd(bool force_fit);
-    void fit_snd_bufs(unsigned int new_max);
+    void fit_snd_bufs(uint32_t new_snd_buf_max);
+    bool rx_tls_msg(struct msghdr *__msg, mem_buf_desc_t *out_buf);
 
     inline struct tcp_seg *get_tcp_seg_cached();
     inline struct tcp_seg *get_tcp_seg_direct();
@@ -570,12 +587,6 @@ private:
     inline void put_tcp_seg_direct(struct tcp_seg *seg);
     inline void return_tcp_segs(struct tcp_seg *seg);
 
-    void queue_rx_ctl_packet(struct tcp_pcb *pcb, mem_buf_desc_t *p_desc);
-    bool process_peer_ctl_packets(xlio_desc_list_t &peer_packets);
-    void process_my_ctl_packets();
-    void process_children_ctl_packets();
-    void process_reuse_ctl_packets();
-    void process_rx_ctl_packets();
     static void put_agent_msg(void *arg);
     bool is_connected_and_ready_to_send();
 
@@ -597,9 +608,10 @@ private:
     timestamps_t m_rx_timestamps;
     tcp_sock_offload_e m_sock_offload;
     tcp_sock_state_e m_sock_state;
+    std::atomic<int32_t> m_snd_buf;
+    uint32_t m_snd_buf_max;
     sockinfo_tcp *m_parent;
     // received packet source (true if its from internal thread)
-    bool m_xlio_thr;
     bool m_b_incoming;
     bool m_b_attached;
     bool m_timer_registered = false;
@@ -645,26 +657,23 @@ private:
     uint32_t m_tcp_seg_in_use;
 
     xlio_desc_list_t m_rx_pkt_ready_list;
-    xlio_desc_list_t m_rx_cb_dropped_list;
-
-    lock_spin_recursive m_rx_ctl_packets_list_lock;
-    tscval_t m_last_syn_tsc;
-    xlio_desc_list_t m_rx_ctl_packets_list;
-    peer_map_t m_rx_peer_packets;
-    xlio_desc_list_t m_rx_ctl_reuse_list;
-    ready_pcb_map_t m_ready_pcbs;
     static const unsigned TX_CONSECUTIVE_EAGAIN_THREASHOLD = 10;
     unsigned m_tx_consecutive_eagain_count;
     bool m_sysvar_rx_poll_on_tx_tcp;
-    uint64_t m_user_huge_page_mask;
     uint16_t m_external_vlan_tag = 0U;
     /*
-     * Storage API
+     * XLIO Ultra API
      * TODO Move the fields to proper cold/hot sections in the final version.
      */
     bool m_b_xlio_socket_dirty = false;
     uintptr_t m_xlio_socket_userdata = 0;
-    poll_group *m_p_group = nullptr;
+    rfs_rule *m_p_rule_extracted = nullptr;
+
+    mem_buf_desc_t *m_store = nullptr;
+    uint32_t m_store_offset = 0;
+
+    // Listen context - allocated only for listen sockets
+    sockinfo_tcp_listen_context *m_listen_ctx = nullptr;
 };
 
 #endif
