@@ -10,7 +10,6 @@
 #include <stddef.h>
 #include <string.h>
 #include <bitset>
-#include <limits.h>
 #include <netinet/in.h>
 #include <linux/if.h>
 #include <sys/types.h>
@@ -18,16 +17,17 @@
 #include <vlogger/vlogger.h>
 #include <core/xlio_extra.h>
 #include <core/util/sock_addr.h>
+#include <core/util/utils.h>
 #include <assert.h>
 #include <atomic>
 
 #define NUM_OF_SUPPORTED_CQS         16
 #define NUM_OF_SUPPORTED_RINGS       16
+#define NUM_OF_SUPPORTED_ENTITY_CTX  16
 #define NUM_OF_SUPPORTED_BPOOLS      4
 #define NUM_OF_SUPPORTED_GLOBALS     1
 #define NUM_OF_SUPPORTED_EPFDS       32
 #define SHMEM_STATS_SIZE(fds_num)    sizeof(sh_mem_t) + (fds_num * sizeof(socket_instance_block_t))
-#define FILE_NAME_MAX_SIZE           (NAME_MAX + 1)
 #define MC_TABLE_SIZE                1024
 #define MAP_SH_MEM(var, sh_stats)    var = (sh_mem_t *)sh_stats
 #define STATS_PUBLISHER_TIMER_PERIOD 10 // publisher will check for stats request every 10 msec
@@ -60,7 +60,7 @@ typedef struct {
 
 typedef enum { e_totals = 1, e_deltas } print_details_mode_t;
 
-typedef enum { e_basic = 1, e_medium, e_full, e_mc_groups, e_netstat_like } view_mode_t;
+typedef enum { e_basic = 1, e_medium, e_full, e_mc_groups, e_netstat_like, e_entctx } view_mode_t;
 
 typedef enum { e_by_pid_str, e_by_app_name, e_by_runn_proccess } proc_ident_mode_t;
 
@@ -86,18 +86,24 @@ extern user_params_t user_params;
 
 // Epoll group stats
 typedef struct {
-    bool enabled;
-    int epfd;
     iomux_func_stats_t stats;
+    int epfd;
+    bool enabled;
+    PADDING(27); // Pad to cache line boundary
 } epoll_stats_t;
 
+CACHELINE_BOUNDARY_SIZE_ASSERT(epoll_stats_t);
+
 // iomux function stat info
 typedef struct {
     iomux_func_stats_t poll;
     iomux_func_stats_t select;
     epoll_stats_t epoll[NUM_OF_SUPPORTED_EPFDS];
+    PADDING(0); // Pad to cache line boundary
 } iomux_stats_t;
 
+CACHELINE_BOUNDARY_SIZE_ASSERT(iomux_stats_t);
+
 // multicast stat info
 typedef struct {
     uint32_t sock_num;
@@ -153,8 +159,12 @@ typedef struct {
     uint32_t n_tls_tx_resync;
     uint32_t n_tls_tx_resync_replay;
     uint32_t n_tls_rx_records;
-    uint32_t n_tls_rx_records_enc;
-    uint32_t n_tls_rx_records_partial;
+    uint32_t n_tls_rx_records_full_enc;
+    uint32_t n_tls_rx_records_head_enc;
+    uint32_t n_tls_rx_records_tail_enc;
+    uint32_t n_tls_rx_records_mix_enc;
+    uint32_t n_tls_rx_records_hw_auth_fail;
+    uint32_t n_tls_rx_records_sw_dec_fail;
     uint32_t n_tls_rx_resync;
 } socket_tls_counters_t;
 #endif /* DEFINED_UTLS */
@@ -204,6 +214,7 @@ struct socket_stats_t {
     uint64_t n_tx_ready_byte_count;
     uint64_t n_rx_ready_byte_count;
     uint32_t n_rx_ready_pkt_count;
+    uint32_t tcp_state; // enum tcp_state
     socket_counters_t counters;
     socket_strq_counters_t strq_counters;
 #ifdef DEFINED_UTLS
@@ -220,8 +231,6 @@ struct socket_stats_t {
     ip_address mc_tx_if;
     int fd;
     uint32_t inode;
-    uint32_t tcp_state; // enum tcp_state
-    uint32_t n_rx_zcopy_pkt_count;
     pid_t threadid_last_rx;
     pid_t threadid_last_tx;
     uint64_t ring_user_id_rx;
@@ -251,8 +260,7 @@ struct socket_stats_t {
         bound_if = connected_ip = mc_tx_if = ip_address(in6addr_any);
         bound_port = connected_port = (in_port_t)0;
         threadid_last_rx = threadid_last_tx = pid_t(0);
-        n_rx_ready_pkt_count = n_rx_ready_byte_count = n_rx_zcopy_pkt_count =
-            n_tx_ready_byte_count = 0;
+        n_rx_ready_pkt_count = n_rx_ready_byte_count = n_tx_ready_byte_count = 0;
         memset(&counters, 0, sizeof(counters));
 #ifdef DEFINED_UTLS
         tls_tx_offload = tls_rx_offload = false;
@@ -308,9 +316,10 @@ typedef struct {
 // CQ stat info
 typedef struct {
     uint64_t n_rx_stride_count;
+    uint64_t n_rx_hw_pkt_drops;
     uint64_t n_rx_packet_count;
     uint64_t n_rx_consumed_rwqe_count;
-    uint64_t n_rx_pkt_drop;
+    uint64_t n_rx_sw_pkt_drops;
     uint64_t n_rx_lro_packets;
     uint64_t n_rx_lro_bytes;
     uint64_t n_rx_gro_packets;
@@ -324,72 +333,96 @@ typedef struct {
 } cq_stats_t;
 
 typedef struct {
-    bool b_enabled;
     cq_stats_t cq_stats;
+    bool b_enabled;
+    PADDING(23); // Pad to cache line boundary
 } cq_instance_block_t;
 
-typedef enum { RING_ETH = 0, RING_TAP } ring_type_t;
-
-static const char *const ring_type_str[] = {"RING_ETH", "RING_TAP"};
+CACHELINE_BOUNDARY_SIZE_ASSERT(cq_instance_block_t);
 
 // Ring stat info
 typedef struct {
     uint64_t n_rx_pkt_count;
     uint64_t n_rx_byte_count;
+    uint64_t n_rx_interrupt_requests;
+    uint64_t n_rx_interrupt_received;
+    uint32_t n_rx_cq_moderation_count;
+    uint32_t n_rx_cq_moderation_period;
+    uint32_t n_rx_tls_contexts;
+    uint32_t n_rx_tls_resyncs;
+    uint32_t n_rx_tls_auth_fail;
+    uint32_t n_rx_zc_migiration_drop;
+    void *p_ring_master;
+
+    // First cache line
+
     uint64_t n_tx_pkt_count;
     uint64_t n_tx_byte_count;
     uint64_t n_tx_retransmits;
-    void *p_ring_master;
-#ifdef DEFINED_UTLS
+    uint64_t n_tx_tso_pkt_count;
+    uint64_t n_tx_tso_byte_count;
+    uint32_t n_tx_num_bufs;
+    uint32_t n_zc_num_bufs;
+    uint64_t n_tx_dropped_wqes;
     uint32_t n_tx_tls_contexts;
-    uint32_t n_rx_tls_contexts;
-#endif /* DEFINED_UTLS */
-    ring_type_t n_type;
-    union {
-        struct {
-            uint64_t n_tx_tso_pkt_count;
-            uint64_t n_tx_tso_byte_count;
-            uint64_t n_rx_interrupt_requests;
-            uint64_t n_rx_interrupt_received;
-            uint32_t n_rx_cq_moderation_count;
-            uint32_t n_rx_cq_moderation_period;
-            uint64_t n_tx_dropped_wqes;
-            uint64_t n_tx_dev_mem_pkt_count;
-            uint64_t n_tx_dev_mem_byte_count;
-            uint64_t n_tx_dev_mem_oob;
-            uint32_t n_tx_dev_mem_allocated;
-            uint32_t n_tx_num_bufs;
-            uint32_t n_zc_num_bufs;
-        } simple;
-        struct {
-            char s_tap_name[IFNAMSIZ];
-            uint32_t n_tap_fd;
-            uint32_t n_rx_buffers;
-            uint32_t n_vf_plugouts;
-        } tap;
-    };
+    uint32_t n_tx_tls_resyncs;
+
+    // Second cache line
+
+    uint64_t n_tx_dev_mem_pkt_count;
+    uint64_t n_tx_dev_mem_byte_count;
+    uint64_t n_tx_dev_mem_oob;
+    uint32_t n_tx_dev_mem_allocated;
+    uint32_t n_rx_steering_rules;
 } ring_stats_t;
 
 typedef struct {
-    bool b_enabled;
     ring_stats_t ring_stats;
+    bool b_enabled;
+    PADDING(31); // Pad to cache line boundary
 } ring_instance_block_t;
 
+CACHELINE_BOUNDARY_SIZE_ASSERT(ring_instance_block_t);
+
+typedef struct {
+    int64_t idle_time;
+    int64_t hit_poll_time;
+    int64_t job_proc_time;
+    uint64_t socket_num_added;
+    uint64_t socket_num_removed;
+    uint32_t listen_rsschild_num;
+    uint32_t job_queue_size_max;
+    uint32_t job_queue_size_acc;
+    uint32_t job_queue_hits;
+} entity_context_stats_t;
+
+typedef struct {
+    entity_context_stats_t entity_ctx_stats;
+    uint32_t tot_socket;
+    bool b_enabled;
+    PADDING(3); // Pad to cache line boundary
+} entity_context_instance_block_t;
+
+CACHELINE_BOUNDARY_SIZE_ASSERT(entity_context_instance_block_t);
+
 // Buffer Pool stat info
 typedef struct {
-    bool is_rx;
-    bool is_tx;
     uint32_t n_buffer_pool_size;
     uint32_t n_buffer_pool_no_bufs;
     uint32_t n_buffer_pool_expands;
     uint32_t n_buffer_pool_created;
+    bool is_rx;
+    bool is_tx;
 } bpool_stats_t;
 
 typedef struct {
-    bool b_enabled;
     bpool_stats_t bpool_stats;
+    bool b_enabled;
+    PADDING(11); // Pad to half cache line boundary
 } bpool_instance_block_t;
 
+BOUNDARY_SIZE_ASSERT(bpool_instance_block_t, CACHELINE_SIZE / 2);
+
 // Global stat info
 typedef struct {
     uint32_t n_tcp_seg_pool_size;
@@ -408,8 +441,8 @@ typedef struct {
 } global_stats_t;
 
 typedef struct {
-    bool b_enabled;
     global_stats_t global_stats;
+    bool b_enabled;
     void init()
     {
         b_enabled = false;
@@ -426,22 +459,23 @@ typedef struct {
 } version_info_t;
 
 typedef struct sh_mem_t {
+    cq_instance_block_t cq_inst_arr[NUM_OF_SUPPORTED_CQS];
+    ring_instance_block_t ring_inst_arr[NUM_OF_SUPPORTED_RINGS];
+    entity_context_instance_block_t ent_ctx_inst_arr[NUM_OF_SUPPORTED_ENTITY_CTX];
+    bpool_instance_block_t bpool_inst_arr[NUM_OF_SUPPORTED_BPOOLS];
+    iomux_stats_t iomux;
+    global_instance_block_t global_inst_arr[NUM_OF_SUPPORTED_GLOBALS];
     int reader_counter; // only copy to shm upon active reader
     version_info_t ver_info;
-    char stats_protocol_ver[32];
     vlog_levels_t log_level;
     uint8_t log_details_level;
     dump_type_t dump;
     int fd_dump;
     vlog_levels_t fd_dump_log_level;
-    cq_instance_block_t cq_inst_arr[NUM_OF_SUPPORTED_CQS];
-    ring_instance_block_t ring_inst_arr[NUM_OF_SUPPORTED_RINGS];
-    bpool_instance_block_t bpool_inst_arr[NUM_OF_SUPPORTED_BPOOLS];
-    global_instance_block_t global_inst_arr[NUM_OF_SUPPORTED_GLOBALS];
     mc_grp_info_t mc_info;
-    iomux_stats_t iomux;
     size_t max_skt_inst_num; // number of elements allocated in 'socket_instance_block_t
                              // skt_inst_arr[]'
+    char stats_protocol_ver[32];
 
     /* IMPORTANT:  MUST BE LAST ENTRY in struct: [0] is the allocation start point for all fd's
      *
@@ -520,6 +554,9 @@ void xlio_stats_instance_remove_ring_block(ring_stats_t *);
 void xlio_stats_instance_create_cq_block(cq_stats_t *);
 void xlio_stats_instance_remove_cq_block(cq_stats_t *);
 
+void xlio_stats_instance_create_ent_ctx_block(entity_context_stats_t *);
+void xlio_stats_instance_remove_ent_ctx_block(entity_context_stats_t *);
+
 void xlio_stats_instance_create_bpool_block(bpool_stats_t *);
 void xlio_stats_instance_remove_bpool_block(bpool_stats_t *);
 
