@@ -9,16 +9,16 @@
 #define UCT_VERSION_H_
 
 #define UCT_VERNO_MAJOR            1
-#define UCT_VERNO_MINOR            19
+#define UCT_VERNO_MINOR            20
 #define UCT_VERNO_PATCH            0
 #define UCT_VERNO_EXTRA            ""
-#define UCT_VERNO_STRING           "1.19.0"
-#define UCT_SCM_VERSION            "13ae265"
-#define UCT_SCM_BRANCH             ""
+#define UCT_VERNO_STRING           "1.20.0"
+#define UCT_SCM_VERSION            "03898fe"
+#define UCT_SCM_BRANCH             "master"
 
 #define UCT_MINOR_BIT              (16UL)
 #define UCT_MAJOR_BIT              (24UL)
-#define UCT_API                    ((1L<<UCT_MAJOR_BIT)|(19L << UCT_MINOR_BIT))
+#define UCT_API                    ((1L<<UCT_MAJOR_BIT)|(20L << UCT_MINOR_BIT))
 
 #define UCT_VERSION(major, minor)  (((major)<<UCT_MAJOR_BIT)|((minor)<<UCT_MINOR_BIT))
 
