@@ -100,7 +100,13 @@ static void nvmet_bdev_ns_enable_integrity(struct nvmet_ns *ns)
 
 #ifdef HAVE_BLK_INTEGRITY_CSUM_CRC64
 	if (bi->csum_type == BLK_INTEGRITY_CSUM_CRC) {
+#ifdef HAVE_BLK_INTEGRITY_METADATA_SIZE
+		/* Newer kernels (v6.17-rc1+): field renamed to metadata_size */
+		ns->metadata_size = bi->metadata_size;
+#else
+		/* Older kernels: field was called tuple_size */
 		ns->metadata_size = bi->tuple_size;
+#endif
 		if (bi->flags & BLK_INTEGRITY_REF_TAG)
 			ns->pi_type = NVME_NS_DPS_PI_TYPE1;
 		else
